@@ -218,6 +218,11 @@ public:
     */
     wxString GetOpenedPageTitle() const;
 
+    /**
+       Returns a pointer to the current parser.
+    */
+    wxHtmlWinParser *GetParser() const;
+
     /**
         Returns the related frame.
     */
