@@ -1,8 +1,7 @@
 /////////////////////////////////////////////////////////////////////////////
-// Name:        tarstrm.cpp
+// Name:        src/common/tarstrm.cpp
 // Purpose:     Streams for Tar files
 // Author:      Mike Wetherell
-// RCS-ID:      $Id: tarstrm.cpp 63302 2010-01-28 21:46:09Z MW $
 // Copyright:   (c) 2004 Mike Wetherell
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -26,7 +25,7 @@
 
 #include "wx/buffer.h"
 #include "wx/datetime.h"
-#include "wx/ptr_scpd.h"
+#include "wx/scopedptr.h"
 #include "wx/filename.h"
 #include "wx/thread.h"
 
@@ -104,9 +103,9 @@ wxTarClassFactory::wxTarClassFactory()
 const wxChar * const *
 wxTarClassFactory::GetProtocols(wxStreamProtocolType type) const
 {
-    static const wxChar *protocols[] = { _T("tar"), NULL };
-    static const wxChar *mimetypes[] = { _T("application/x-tar"), NULL };
-    static const wxChar *fileexts[]  = { _T(".tar"), NULL };
+    static const wxChar *protocols[] = { wxT("tar"), NULL };
+    static const wxChar *mimetypes[] = { wxT("application/x-tar"), NULL };
+    static const wxChar *fileexts[]  = { wxT(".tar"), NULL };
     static const wxChar *empty[]     = { NULL };
 
     switch (type) {
@@ -161,23 +160,23 @@ wxDEFINE_SCOPED_PTR_TYPE(wxTarHeaderBlock)
 // A table giving the field names and offsets in a tar header block
 const wxTarField wxTarHeaderBlock::fields[] =
 {
-    { _T("name"), 0 },       // 100
-    { _T("mode"), 100 },     // 8
-    { _T("uid"), 108 },      // 8
-    { _T("gid"), 116 },      // 8
-    { _T("size"), 124 },     // 12
-    { _T("mtime"), 136 },    // 12
-    { _T("chksum"), 148 },   // 8
-    { _T("typeflag"), 156 }, // 1
-    { _T("linkname"), 157 }, // 100
-    { _T("magic"), 257 },    // 6
-    { _T("version"), 263 },  // 2
-    { _T("uname"), 265 },    // 32
-    { _T("gname"), 297 },    // 32
-    { _T("devmajor"), 329 }, // 8
-    { _T("devminor"), 337 }, // 8
-    { _T("prefix"), 345 },   // 155
-    { _T("unused"), 500 },   // 12
+    { wxT("name"), 0 },       // 100
+    { wxT("mode"), 100 },     // 8
+    { wxT("uid"), 108 },      // 8
+    { wxT("gid"), 116 },      // 8
+    { wxT("size"), 124 },     // 12
+    { wxT("mtime"), 136 },    // 12
+    { wxT("chksum"), 148 },   // 8
+    { wxT("typeflag"), 156 }, // 1
+    { wxT("linkname"), 157 }, // 100
+    { wxT("magic"), 257 },    // 6
+    { wxT("version"), 263 },  // 2
+    { wxT("uname"), 265 },    // 32
+    { wxT("gname"), 297 },    // 32
+    { wxT("devmajor"), 329 }, // 8
+    { wxT("devminor"), 337 }, // 8
+    { wxT("prefix"), 345 },   // 155
+    { wxT("unused"), 500 },   // 12
     { NULL, TAR_BLOCKSIZE }
 };
 
@@ -291,7 +290,10 @@ bool wxTarHeaderBlock::SetPath(const wxString& name, wxMBConv& conv)
         size_t len = name.length();
         wxCharBuffer approx(len);
         for (size_t i = 0; i < len; i++)
-            approx.data()[i] = name[i] & ~0x7F ? '_' : name[i];
+        {
+            wxChar c = name[i];
+            approx.data()[i] = c & ~0x7F ? '_' : c;
+        }
         nameBuf = approx;
     }
 
@@ -536,15 +538,15 @@ wxString wxTarEntry::GetName(wxPathFormat format /*=wxPATH_NATIVE*/) const
     switch (wxFileName::GetFormat(format)) {
         case wxPATH_DOS:
         {
-            wxString name(isDir ? m_Name + _T("\\") : m_Name);
+            wxString name(isDir ? m_Name + wxT("\\") : m_Name);
             for (size_t i = 0; i < name.length(); i++)
-                if (name[i] == _T('/'))
-                    name[i] = _T('\\');
+                if (name[i] == wxT('/'))
+                    name[i] = wxT('\\');
             return name;
         }
 
         case wxPATH_UNIX:
-            return isDir ? m_Name + _T("/") : m_Name;
+            return isDir ? m_Name + wxT("/") : m_Name;
 
         default:
             ;
@@ -591,9 +593,9 @@ wxString wxTarEntry::GetInternalName(const wxString& name,
 
     while (!internal.empty() && *internal.begin() == '/')
         internal.erase(0, 1);
-    while (!internal.empty() && internal.compare(0, 2, _T("./")) == 0)
+    while (!internal.empty() && internal.compare(0, 2, wxT("./")) == 0)
         internal.erase(0, 2);
-    if (internal == _T(".") || internal == _T(".."))
+    if (internal == wxT(".") || internal == wxT(".."))
         internal = wxEmptyString;
 
     return internal;
@@ -692,9 +694,9 @@ wxTarEntry *wxTarInputStream::GetNextEntry()
 
     entry->SetOffset(m_offset);
 
-    entry->SetDateTime(GetHeaderDate(_T("mtime")));
-    entry->SetAccessTime(GetHeaderDate(_T("atime")));
-    entry->SetCreateTime(GetHeaderDate(_T("ctime")));
+    entry->SetDateTime(GetHeaderDate(wxT("mtime")));
+    entry->SetAccessTime(GetHeaderDate(wxT("atime")));
+    entry->SetCreateTime(GetHeaderDate(wxT("ctime")));
 
     entry->SetTypeFlag(*m_hdr->Get(TAR_TYPEFLAG));
     bool isDir = entry->IsDir();
@@ -790,7 +792,9 @@ wxStreamError wxTarInputStream::ReadHeaders()
     while (!done) {
         m_hdr->Read(*m_parent_i_stream);
         if (m_parent_i_stream->Eof())
+        {
             wxLogError(_("incomplete header block in tar"));
+        }
         if (!*m_parent_i_stream)
             return wxSTREAM_READ_ERROR;
         m_offset += TAR_BLOCKSIZE;
@@ -863,7 +867,7 @@ wxString wxTarInputStream::GetHeaderPath() const
 {
     wxString path;
 
-    if ((path = GetExtendedHeader(_T("path"))) != wxEmptyString)
+    if ((path = GetExtendedHeader(wxT("path"))) != wxEmptyString)
         return path;
 
     path = wxString(m_hdr->Get(TAR_NAME), GetConv());
@@ -871,7 +875,7 @@ wxString wxTarInputStream::GetHeaderPath() const
         return path;
 
     const char *prefix = m_hdr->Get(TAR_PREFIX);
-    return *prefix ? wxString(prefix, GetConv()) + _T("/") + path : path;
+    return *prefix ? wxString(prefix, GetConv()) + wxT("/") + path : path;
 }
 
 wxDateTime wxTarInputStream::GetHeaderDate(const wxString& key) const
@@ -885,7 +889,7 @@ wxDateTime wxTarInputStream::GetHeaderDate(const wxString& key) const
         return ll;
     }
 
-    if (key == _T("mtime"))
+    if (key == wxT("mtime"))
         return wxLongLong(m_hdr->GetOctal(TAR_MTIME)) * 1000L;
 
     return wxDateTime();
@@ -897,8 +901,8 @@ wxTarNumber wxTarInputStream::GetHeaderNumber(int id) const
 
     if ((value = GetExtendedHeader(m_hdr->Name(id))) != wxEmptyString) {
         wxTarNumber n = 0;
-        const wxChar *p = value;
-        while (*p == ' ')
+        wxString::const_iterator p = value.begin();
+        while (p != value.end() && *p == ' ')
             p++;
         while (isdigit(*p))
             n = n * 10 + (*p++ - '0');
@@ -942,7 +946,7 @@ bool wxTarInputStream::ReadExtendedHeader(wxTarHeaderRecords*& recs)
     size_t recPos, recSize;
     bool ok = true;
 
-    for (recPos = 0; recPos < len; recPos += recSize) {
+    for (recPos = 0; recPos < len && ok; recPos += recSize) {
         char *pRec = buf.data() + recPos;
         char *p = pRec;
 
@@ -1037,7 +1041,9 @@ size_t wxTarInputStream::OnSysRead(void *buffer, size_t size)
     } else if (!m_parent_i_stream->IsOk()) {
         // any other error will have been reported by the underlying stream
         if (m_parent_i_stream->Eof())
+        {
             wxLogError(_("unexpected end of file"));
+        }
         m_lasterror = wxSTREAM_READ_ERROR;
     }
 
@@ -1082,12 +1088,12 @@ void wxTarOutputStream::Init(wxTarFormat format)
     m_extendedHdr = NULL;
     m_extendedSize = 0;
     m_lasterror = m_parent_o_stream->GetLastError();
+    m_endrecWritten = false;
 }
 
 wxTarOutputStream::~wxTarOutputStream()
 {
-    if (m_tarsize)
-        Close();
+    Close();
     delete m_hdr;
     delete m_hdr2;
     delete [] m_extendedHdr;
@@ -1206,7 +1212,7 @@ bool wxTarOutputStream::CloseEntry()
 
 bool wxTarOutputStream::Close()
 {
-    if (!CloseEntry())
+    if (!CloseEntry() || (m_tarsize == 0 && m_endrecWritten))
         return false;
 
     memset(m_hdr, 0, sizeof(*m_hdr));
@@ -1218,6 +1224,7 @@ bool wxTarOutputStream::Close()
     m_tarsize = 0;
     m_tarstart = wxInvalidOffset;
     m_lasterror = m_parent_o_stream->GetLastError();
+    m_endrecWritten = true;
     return IsOk();
 }
 
@@ -1235,11 +1242,11 @@ bool wxTarOutputStream::WriteHeaders(wxTarEntry& entry)
         entry.SetSize(0);
     m_large = !SetHeaderNumber(TAR_SIZE, entry.GetSize());
 
-    SetHeaderDate(_T("mtime"), entry.GetDateTime());
+    SetHeaderDate(wxT("mtime"), entry.GetDateTime());
     if (entry.GetAccessTime().IsValid())
-        SetHeaderDate(_T("atime"), entry.GetAccessTime());
+        SetHeaderDate(wxT("atime"), entry.GetAccessTime());
     if (entry.GetCreateTime().IsValid())
-        SetHeaderDate(_T("ctime"), entry.GetCreateTime());
+        SetHeaderDate(wxT("ctime"), entry.GetCreateTime());
 
     *m_hdr->Get(TAR_TYPEFLAG) = char(entry.GetTypeFlag());
 
@@ -1274,7 +1281,7 @@ bool wxTarOutputStream::WriteHeaders(wxTarEntry& entry)
         // an old tar that doesn't understand extended headers will
         // extract it as a file, so give these fields reasonable values
         // so that the user will have access to read and remove it.
-        m_hdr2->SetPath(PaxHeaderPath(_T("%d/PaxHeaders.%p/%f"),
+        m_hdr2->SetPath(PaxHeaderPath(wxT("%d/PaxHeaders.%p/%f"),
                                       entry.GetName(wxPATH_UNIX)), GetConv());
         m_hdr2->SetOctal(TAR_MODE, 0600);
         strcpy(m_hdr2->Get(TAR_UID), m_hdr->Get(TAR_UID));
@@ -1324,12 +1331,12 @@ bool wxTarOutputStream::WriteHeaders(wxTarEntry& entry)
 wxString wxTarOutputStream::PaxHeaderPath(const wxString& format,
                                           const wxString& path)
 {
-    wxString d = path.BeforeLast(_T('/'));
-    wxString f = path.AfterLast(_T('/'));
+    wxString d = path.BeforeLast(wxT('/'));
+    wxString f = path.AfterLast(wxT('/'));
     wxString ret;
 
     if (d.empty())
-        d = _T(".");
+        d = wxT(".");
 
     ret.reserve(format.length() + path.length() + 16);
 
@@ -1341,11 +1348,11 @@ wxString wxTarOutputStream::PaxHeaderPath(const wxString& format,
         if (end == wxString::npos || end + 1 >= format.length())
             break;
         ret << format.substr(begin, end - begin);
-        switch (format[end + 1]) {
+        switch ( format[end + 1].GetValue() ) {
             case 'd': ret << d; break;
             case 'f': ret << f; break;
             case 'p': ret << wxGetProcessId(); break;
-            case '%': ret << _T("%"); break;
+            case '%': ret << wxT("%"); break;
         }
         begin = end + 2;
     }
@@ -1390,7 +1397,7 @@ bool wxTarOutputStream::ModifyHeader()
 void wxTarOutputStream::SetHeaderPath(const wxString& name)
 {
     if (!m_hdr->SetPath(name, GetConv()) || (m_pax && !name.IsAscii()))
-        SetExtendedHeader(_T("path"), name);
+        SetExtendedHeader(wxT("path"), name);
 }
 
 bool wxTarOutputStream::SetHeaderNumber(int id, wxTarNumber n)
@@ -1416,13 +1423,13 @@ void wxTarOutputStream::SetHeaderDate(const wxString& key,
     wxLongLong ll = datetime.IsValid() ? datetime.GetValue() : wxLongLong(0);
     wxLongLong secs = ll / 1000L;
 
-    if (key != _T("mtime")
+    if (key != wxT("mtime")
         || !m_hdr->SetOctal(TAR_MTIME, wxTarNumber(secs.GetValue()))
         || secs <= 0 || secs >= 0x7fffffff)
     {
         wxString str;
         if (ll >= LONG_MIN && ll <= LONG_MAX) {
-            str.Printf(_T("%g"), ll.ToLong() / 1000.0);
+            str.Printf(wxT("%g"), ll.ToLong() / 1000.0);
         } else {
             str = ll.ToString();
             str.insert(str.end() - 3, '.');
@@ -1435,11 +1442,16 @@ void wxTarOutputStream::SetExtendedHeader(const wxString& key,
                                           const wxString& value)
 {
     if (m_pax) {
+#if wxUSE_UNICODE
+        const wxCharBuffer utf_key = key.utf8_str();
+        const wxCharBuffer utf_value = value.utf8_str();
+#else
         const wxWX2WCbuf wide_key = key.wc_str(GetConv());
         const wxCharBuffer utf_key = wxConvUTF8.cWC2MB(wide_key);
 
         const wxWX2WCbuf wide_value = value.wc_str(GetConv());
         const wxCharBuffer utf_value = wxConvUTF8.cWC2MB(wide_value);
+#endif // wxUSE_UNICODE/!wxUSE_UNICODE
 
         // a small buffer to format the length field in
         char buf[32];
@@ -1479,7 +1491,7 @@ void wxTarOutputStream::SetExtendedHeader(const wxString& key,
     else {
         // if not pax then make a list of fields to report as errors
         if (!m_badfit.empty())
-            m_badfit += _T(", ");
+            m_badfit += wxT(", ");
         m_badfit += key;
     }
 }
