@@ -4,7 +4,6 @@
 // Author:      Julian Smart
 // Modified by: VZ at 11.12.99 (wxScrollableToolBar split off)
 // Created:     04/01/98
-// RCS-ID:      $Id: tbarbase.cpp 42840 2006-10-31 13:09:08Z VZ $
 // Copyright:   (c) Julian Smart
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -32,9 +31,14 @@
     #include "wx/control.h"
     #include "wx/frame.h"
     #include "wx/settings.h"
-    #include "wx/image.h"
+    #if WXWIN_COMPATIBILITY_2_8
+        #include "wx/image.h"
+    #endif // WXWIN_COMPATIBILITY_2_8
+    #include "wx/menu.h"
 #endif
 
+extern WXDLLEXPORT_DATA(const char) wxToolBarNameStr[] = "toolbar";
+
 // ----------------------------------------------------------------------------
 // wxWidgets macros
 // ----------------------------------------------------------------------------
@@ -56,6 +60,17 @@ WX_DEFINE_LIST(wxToolBarToolsList)
 
 IMPLEMENT_DYNAMIC_CLASS(wxToolBarToolBase, wxObject)
 
+wxToolBarToolBase::~wxToolBarToolBase()
+{
+#if wxUSE_MENUS
+    delete m_dropdownMenu;
+#endif
+
+    if ( IsControl() )
+        GetControl()->Destroy();
+}
+
+
 bool wxToolBarToolBase::Enable(bool enable)
 {
     if ( m_enabled == enable )
@@ -68,7 +83,7 @@ bool wxToolBarToolBase::Enable(bool enable)
 
 bool wxToolBarToolBase::Toggle(bool toggle)
 {
-    wxASSERT_MSG( CanBeToggled(), _T("can't toggle this tool") );
+    wxASSERT_MSG( CanBeToggled(), wxT("can't toggle this tool") );
 
     if ( m_toggled == toggle )
         return false;
@@ -109,6 +124,16 @@ bool wxToolBarToolBase::SetLongHelp(const wxString& help)
     return true;
 }
 
+
+#if wxUSE_MENUS
+void wxToolBarToolBase::SetDropdownMenu(wxMenu* menu)
+{
+    delete m_dropdownMenu;
+    m_dropdownMenu = menu;
+}
+#endif
+
+
 // ----------------------------------------------------------------------------
 // wxToolBarBase adding/deleting items
 // ----------------------------------------------------------------------------
@@ -132,7 +157,7 @@ void wxToolBarBase::FixupStyle()
     }
 }
 
-wxToolBarToolBase *wxToolBarBase::DoAddTool(int id,
+wxToolBarToolBase *wxToolBarBase::DoAddTool(int toolid,
                                             const wxString& label,
                                             const wxBitmap& bitmap,
                                             const wxBitmap& bmpDisabled,
@@ -144,12 +169,12 @@ wxToolBarToolBase *wxToolBarBase::DoAddTool(int id,
                                             wxCoord WXUNUSED(yPos))
 {
     InvalidateBestSize();
-    return InsertTool(GetToolsCount(), id, label, bitmap, bmpDisabled,
+    return InsertTool(GetToolsCount(), toolid, label, bitmap, bmpDisabled,
                       kind, shortHelp, longHelp, clientData);
 }
 
 wxToolBarToolBase *wxToolBarBase::InsertTool(size_t pos,
-                                             int id,
+                                             int toolid,
                                              const wxString& label,
                                              const wxBitmap& bitmap,
                                              const wxBitmap& bmpDisabled,
@@ -158,20 +183,11 @@ wxToolBarToolBase *wxToolBarBase::InsertTool(size_t pos,
                                              const wxString& longHelp,
                                              wxObject *clientData)
 {
-    wxCHECK_MSG( pos <= GetToolsCount(), (wxToolBarToolBase *)NULL,
-                 _T("invalid position in wxToolBar::InsertTool()") );
+    wxCHECK_MSG( pos <= GetToolsCount(), NULL,
+                 wxT("invalid position in wxToolBar::InsertTool()") );
 
-    wxToolBarToolBase *tool = CreateTool(id, label, bitmap, bmpDisabled, kind,
-                                         clientData, shortHelp, longHelp);
-
-    if ( !InsertTool(pos, tool) )
-    {
-        delete tool;
-
-        return NULL;
-    }
-
-    return tool;
+    return DoInsertNewTool(pos, CreateTool(toolid, label, bitmap, bmpDisabled, kind,
+                                           clientData, shortHelp, longHelp));
 }
 
 wxToolBarToolBase *wxToolBarBase::AddTool(wxToolBarToolBase *tool)
@@ -182,8 +198,8 @@ wxToolBarToolBase *wxToolBarBase::AddTool(wxToolBarToolBase *tool)
 wxToolBarToolBase *
 wxToolBarBase::InsertTool(size_t pos, wxToolBarToolBase *tool)
 {
-    wxCHECK_MSG( pos <= GetToolsCount(), (wxToolBarToolBase *)NULL,
-                 _T("invalid position in wxToolBar::InsertTool()") );
+    wxCHECK_MSG( pos <= GetToolsCount(), NULL,
+                 wxT("invalid position in wxToolBar::InsertTool()") );
 
     if ( !tool || !DoInsertTool(pos, tool) )
     {
@@ -191,39 +207,32 @@ wxToolBarBase::InsertTool(size_t pos, wxToolBarToolBase *tool)
     }
 
     m_tools.Insert(pos, tool);
+    tool->Attach(this);
 
     return tool;
 }
 
-wxToolBarToolBase *wxToolBarBase::AddControl(wxControl *control)
+wxToolBarToolBase *
+wxToolBarBase::AddControl(wxControl *control, const wxString& label)
 {
-    return InsertControl(GetToolsCount(), control);
+    return InsertControl(GetToolsCount(), control, label);
 }
 
-wxToolBarToolBase *wxToolBarBase::InsertControl(size_t pos, wxControl *control)
+wxToolBarToolBase *
+wxToolBarBase::InsertControl(size_t pos,
+                             wxControl *control,
+                             const wxString& label)
 {
-    wxCHECK_MSG( control, (wxToolBarToolBase *)NULL,
-                 _T("toolbar: can't insert NULL control") );
+    wxCHECK_MSG( control, NULL,
+                 wxT("toolbar: can't insert NULL control") );
 
-    wxCHECK_MSG( control->GetParent() == this, (wxToolBarToolBase *)NULL,
-                 _T("control must have toolbar as parent") );
+    wxCHECK_MSG( control->GetParent() == this, NULL,
+                 wxT("control must have toolbar as parent") );
 
-    wxCHECK_MSG( pos <= GetToolsCount(), (wxToolBarToolBase *)NULL,
-                 _T("invalid position in wxToolBar::InsertControl()") );
-
-    wxToolBarToolBase *tool = CreateTool(control);
-
-    if ( !InsertTool(pos, tool) )
-    {
-        delete tool;
-
-        return NULL;
-    }
-
-    return tool;
+    return DoInsertNewTool(pos, CreateTool(control, label));
 }
 
-wxControl *wxToolBarBase::FindControl( int id )
+wxControl *wxToolBarBase::FindControl( int toolid )
 {
     for ( wxToolBarToolsList::compatibility_iterator node = m_tools.GetFirst();
           node;
@@ -236,9 +245,9 @@ wxControl *wxToolBarBase::FindControl( int id )
 
             if ( !control )
             {
-                wxFAIL_MSG( _T("NULL control in toolbar?") );
+                wxFAIL_MSG( wxT("NULL control in toolbar?") );
             }
-            else if ( control->GetId() == id )
+            else if ( control->GetId() == toolid )
             {
                 // found
                 return control;
@@ -256,34 +265,38 @@ wxToolBarToolBase *wxToolBarBase::AddSeparator()
 
 wxToolBarToolBase *wxToolBarBase::InsertSeparator(size_t pos)
 {
-    wxCHECK_MSG( pos <= GetToolsCount(), (wxToolBarToolBase *)NULL,
-                 _T("invalid position in wxToolBar::InsertSeparator()") );
-
-    wxToolBarToolBase *tool = CreateTool(wxID_SEPARATOR,
-                                         wxEmptyString,
-                                         wxNullBitmap, wxNullBitmap,
-                                         wxITEM_SEPARATOR, (wxObject *)NULL,
-                                         wxEmptyString, wxEmptyString);
-
-    if ( !tool || !DoInsertTool(pos, tool) )
-    {
-        delete tool;
-
-        return NULL;
-    }
-
-    m_tools.Insert(pos, tool);
-
-    return tool;
+    return DoInsertNewTool(pos, CreateSeparator());
 }
 
-wxToolBarToolBase *wxToolBarBase::RemoveTool(int id)
+wxToolBarToolBase *wxToolBarBase::AddStretchableSpace()
+{
+    return InsertStretchableSpace(GetToolsCount());
+}
+
+wxToolBarToolBase *wxToolBarBase::InsertStretchableSpace(size_t pos)
+{
+    wxToolBarToolBase * const tool = CreateSeparator();
+    if ( tool )
+    {
+        // this is a hack but we know that all the current implementations
+        // don't really use the tool when it's created, they will do it
+        // InsertTool() at earliest and maybe even in Realize() much later
+        //
+        // so we can create the tool as a plain separator and mark it as being
+        // a stretchable space later
+        tool->MakeStretchable();
+    }
+
+    return DoInsertNewTool(pos, tool);
+}
+
+wxToolBarToolBase *wxToolBarBase::RemoveTool(int toolid)
 {
     size_t pos = 0;
     wxToolBarToolsList::compatibility_iterator node;
     for ( node = m_tools.GetFirst(); node; node = node->GetNext() )
     {
-        if ( node->GetData()->GetId() == id )
+        if ( node->GetData()->GetId() == toolid )
             break;
 
         pos++;
@@ -293,24 +306,26 @@ wxToolBarToolBase *wxToolBarBase::RemoveTool(int id)
     {
         // don't give any error messages - sometimes we might call RemoveTool()
         // without knowing whether the tool is or not in the toolbar
-        return (wxToolBarToolBase *)NULL;
+        return NULL;
     }
 
     wxToolBarToolBase *tool = node->GetData();
+    wxCHECK_MSG( tool, NULL, "NULL tool in the tools list?" );
+
     if ( !DoDeleteTool(pos, tool) )
-    {
-        return (wxToolBarToolBase *)NULL;
-    }
+        return NULL;
 
     m_tools.Erase(node);
 
+    tool->Detach();
+
     return tool;
 }
 
 bool wxToolBarBase::DeleteToolByPos(size_t pos)
 {
     wxCHECK_MSG( pos < GetToolsCount(), false,
-                 _T("invalid position in wxToolBar::DeleteToolByPos()") );
+                 wxT("invalid position in wxToolBar::DeleteToolByPos()") );
 
     wxToolBarToolsList::compatibility_iterator node = m_tools.Item(pos);
 
@@ -325,13 +340,13 @@ bool wxToolBarBase::DeleteToolByPos(size_t pos)
     return true;
 }
 
-bool wxToolBarBase::DeleteTool(int id)
+bool wxToolBarBase::DeleteTool(int toolid)
 {
     size_t pos = 0;
     wxToolBarToolsList::compatibility_iterator node;
     for ( node = m_tools.GetFirst(); node; node = node->GetNext() )
     {
-        if ( node->GetData()->GetId() == id )
+        if ( node->GetData()->GetId() == toolid )
             break;
 
         pos++;
@@ -348,16 +363,16 @@ bool wxToolBarBase::DeleteTool(int id)
     return true;
 }
 
-wxToolBarToolBase *wxToolBarBase::FindById(int id) const
+wxToolBarToolBase *wxToolBarBase::FindById(int toolid) const
 {
-    wxToolBarToolBase *tool = (wxToolBarToolBase *)NULL;
+    wxToolBarToolBase *tool = NULL;
 
     for ( wxToolBarToolsList::compatibility_iterator node = m_tools.GetFirst();
           node;
           node = node->GetNext() )
     {
         tool = node->GetData();
-        if ( tool->GetId() == id )
+        if ( tool->GetId() == toolid )
         {
             // found
             break;
@@ -371,13 +386,13 @@ wxToolBarToolBase *wxToolBarBase::FindById(int id) const
 
 void wxToolBarBase::UnToggleRadioGroup(wxToolBarToolBase *tool)
 {
-    wxCHECK_RET( tool, _T("NULL tool in wxToolBarTool::UnToggleRadioGroup") );
+    wxCHECK_RET( tool, wxT("NULL tool in wxToolBarTool::UnToggleRadioGroup") );
 
     if ( !tool->IsButton() || tool->GetKind() != wxITEM_RADIO )
         return;
 
     wxToolBarToolsList::compatibility_iterator node = m_tools.Find(tool);
-    wxCHECK_RET( node, _T("invalid tool in wxToolBarTool::UnToggleRadioGroup") );
+    wxCHECK_RET( node, wxT("invalid tool in wxToolBarTool::UnToggleRadioGroup") );
 
     wxToolBarToolsList::compatibility_iterator nodeNext = node->GetNext();
     while ( nodeNext )
@@ -420,8 +435,34 @@ void wxToolBarBase::ClearTools()
     }
 }
 
+void wxToolBarBase::AdjustToolBitmapSize()
+{
+    const wxSize sizeOrig(m_defaultWidth, m_defaultHeight);
+
+    wxSize sizeActual(sizeOrig);
+
+    for ( wxToolBarToolsList::const_iterator i = m_tools.begin();
+          i != m_tools.end();
+          ++i )
+    {
+        const wxBitmap& bmp = (*i)->GetNormalBitmap();
+        if ( bmp.IsOk() )
+            sizeActual.IncTo(bmp.GetScaledSize());
+    }
+
+    if ( sizeActual != sizeOrig )
+        SetToolBitmapSize(sizeActual);
+}
+
 bool wxToolBarBase::Realize()
 {
+    // check if we have anything to do
+    if ( m_tools.empty() )
+        return false;
+
+    // make sure tool size is large enough for all bitmaps to fit in
+    AdjustToolBitmapSize();
+
     return true;
 }
 
@@ -442,9 +483,9 @@ wxToolBarBase::~wxToolBarBase()
 // wxToolBarBase tools state
 // ----------------------------------------------------------------------------
 
-void wxToolBarBase::EnableTool(int id, bool enable)
+void wxToolBarBase::EnableTool(int toolid, bool enable)
 {
-    wxToolBarToolBase *tool = FindById(id);
+    wxToolBarToolBase *tool = FindById(toolid);
     if ( tool )
     {
         if ( tool->Enable(enable) )
@@ -454,9 +495,9 @@ void wxToolBarBase::EnableTool(int id, bool enable)
     }
 }
 
-void wxToolBarBase::ToggleTool(int id, bool toggle)
+void wxToolBarBase::ToggleTool(int toolid, bool toggle)
 {
-    wxToolBarToolBase *tool = FindById(id);
+    wxToolBarToolBase *tool = FindById(toolid);
     if ( tool && tool->CanBeToggled() )
     {
         if ( tool->Toggle(toggle) )
@@ -467,9 +508,9 @@ void wxToolBarBase::ToggleTool(int id, bool toggle)
     }
 }
 
-void wxToolBarBase::SetToggle(int id, bool toggle)
+void wxToolBarBase::SetToggle(int toolid, bool toggle)
 {
-    wxToolBarToolBase *tool = FindById(id);
+    wxToolBarToolBase *tool = FindById(toolid);
     if ( tool )
     {
         if ( tool->SetToggle(toggle) )
@@ -479,48 +520,48 @@ void wxToolBarBase::SetToggle(int id, bool toggle)
     }
 }
 
-void wxToolBarBase::SetToolShortHelp(int id, const wxString& help)
+void wxToolBarBase::SetToolShortHelp(int toolid, const wxString& help)
 {
-    wxToolBarToolBase *tool = FindById(id);
+    wxToolBarToolBase *tool = FindById(toolid);
     if ( tool )
     {
         (void)tool->SetShortHelp(help);
     }
 }
 
-void wxToolBarBase::SetToolLongHelp(int id, const wxString& help)
+void wxToolBarBase::SetToolLongHelp(int toolid, const wxString& help)
 {
-    wxToolBarToolBase *tool = FindById(id);
+    wxToolBarToolBase *tool = FindById(toolid);
     if ( tool )
     {
         (void)tool->SetLongHelp(help);
     }
 }
 
-wxObject *wxToolBarBase::GetToolClientData(int id) const
+wxObject *wxToolBarBase::GetToolClientData(int toolid) const
 {
-    wxToolBarToolBase *tool = FindById(id);
+    wxToolBarToolBase *tool = FindById(toolid);
 
-    return tool ? tool->GetClientData() : (wxObject *)NULL;
+    return tool ? tool->GetClientData() : NULL;
 }
 
-void wxToolBarBase::SetToolClientData(int id, wxObject *clientData)
+void wxToolBarBase::SetToolClientData(int toolid, wxObject *clientData)
 {
-    wxToolBarToolBase *tool = FindById(id);
+    wxToolBarToolBase *tool = FindById(toolid);
 
-    wxCHECK_RET( tool, _T("no such tool in wxToolBar::SetToolClientData") );
+    wxCHECK_RET( tool, wxT("no such tool in wxToolBar::SetToolClientData") );
 
     tool->SetClientData(clientData);
 }
 
-int wxToolBarBase::GetToolPos(int id) const
+int wxToolBarBase::GetToolPos(int toolid) const
 {
     size_t pos = 0;
     wxToolBarToolsList::compatibility_iterator node;
 
     for ( node = m_tools.GetFirst(); node; node = node->GetNext() )
     {
-        if ( node->GetData()->GetId() == id )
+        if ( node->GetData()->GetId() == toolid )
             return pos;
 
         pos++;
@@ -529,34 +570,34 @@ int wxToolBarBase::GetToolPos(int id) const
     return wxNOT_FOUND;
 }
 
-bool wxToolBarBase::GetToolState(int id) const
+bool wxToolBarBase::GetToolState(int toolid) const
 {
-    wxToolBarToolBase *tool = FindById(id);
-    wxCHECK_MSG( tool, false, _T("no such tool") );
+    wxToolBarToolBase *tool = FindById(toolid);
+    wxCHECK_MSG( tool, false, wxT("no such tool") );
 
     return tool->IsToggled();
 }
 
-bool wxToolBarBase::GetToolEnabled(int id) const
+bool wxToolBarBase::GetToolEnabled(int toolid) const
 {
-    wxToolBarToolBase *tool = FindById(id);
-    wxCHECK_MSG( tool, false, _T("no such tool") );
+    wxToolBarToolBase *tool = FindById(toolid);
+    wxCHECK_MSG( tool, false, wxT("no such tool") );
 
     return tool->IsEnabled();
 }
 
-wxString wxToolBarBase::GetToolShortHelp(int id) const
+wxString wxToolBarBase::GetToolShortHelp(int toolid) const
 {
-    wxToolBarToolBase *tool = FindById(id);
-    wxCHECK_MSG( tool, wxEmptyString, _T("no such tool") );
+    wxToolBarToolBase *tool = FindById(toolid);
+    wxCHECK_MSG( tool, wxEmptyString, wxT("no such tool") );
 
     return tool->GetShortHelp();
 }
 
-wxString wxToolBarBase::GetToolLongHelp(int id) const
+wxString wxToolBarBase::GetToolLongHelp(int toolid) const
 {
-    wxToolBarToolBase *tool = FindById(id);
-    wxCHECK_MSG( tool, wxEmptyString, _T("no such tool") );
+    wxToolBarToolBase *tool = FindById(toolid);
+    wxCHECK_MSG( tool, wxEmptyString, wxT("no such tool") );
 
     return tool->GetLongHelp();
 }
@@ -576,14 +617,20 @@ void wxToolBarBase::SetRows(int WXUNUSED(nRows))
     // nothing
 }
 
+bool wxToolBarBase::IsVertical() const
+{
+    return HasFlag(wxTB_LEFT | wxTB_RIGHT);
+}
+
+
 // ----------------------------------------------------------------------------
 // event processing
 // ----------------------------------------------------------------------------
 
 // Only allow toggle if returns true
-bool wxToolBarBase::OnLeftClick(int id, bool toggleDown)
+bool wxToolBarBase::OnLeftClick(int toolid, bool toggleDown)
 {
-    wxCommandEvent event(wxEVT_COMMAND_TOOL_CLICKED, id);
+    wxCommandEvent event(wxEVT_TOOL, toolid);
     event.SetEventObject(this);
 
     // we use SetInt() to make wxCommandEvent::IsChecked() return toggleDown
@@ -593,42 +640,48 @@ bool wxToolBarBase::OnLeftClick(int id, bool toggleDown)
     event.SetExtraLong((long)toggleDown);
 
     // Send events to this toolbar instead (and thence up the window hierarchy)
-    GetEventHandler()->ProcessEvent(event);
+    HandleWindowEvent(event);
 
     return true;
 }
 
 // Call when right button down.
-void wxToolBarBase::OnRightClick(int id,
+void wxToolBarBase::OnRightClick(int toolid,
                                  long WXUNUSED(x),
                                  long WXUNUSED(y))
 {
-    wxCommandEvent event(wxEVT_COMMAND_TOOL_RCLICKED, id);
+    wxCommandEvent event(wxEVT_TOOL_RCLICKED, toolid);
     event.SetEventObject(this);
-    event.SetInt(id);
+    event.SetInt(toolid);
 
     GetEventHandler()->ProcessEvent(event);
 }
 
 // Called when the mouse cursor enters a tool bitmap (no button pressed).
 // Argument is wxID_ANY if mouse is exiting the toolbar.
-// Note that for this event, the id of the window is used,
+// Note that for this event, the toolid of the window is used,
 // and the integer parameter of wxCommandEvent is used to retrieve
-// the tool id.
-void wxToolBarBase::OnMouseEnter(int id)
+// the tool toolid.
+void wxToolBarBase::OnMouseEnter(int toolid)
 {
-    wxCommandEvent event(wxEVT_COMMAND_TOOL_ENTER, GetId());
+    wxCommandEvent event(wxEVT_TOOL_ENTER, GetId());
     event.SetEventObject(this);
-    event.SetInt(id);
+    event.SetInt(toolid);
 
     wxFrame *frame = wxDynamicCast(GetParent(), wxFrame);
-    if( frame )
+    if ( frame )
     {
         wxString help;
-        wxToolBarToolBase* tool = id == wxID_ANY ? (wxToolBarToolBase*)NULL : FindById(id);
-        if(tool)
-            help = tool->GetLongHelp();
-        frame->DoGiveHelp( help, id != wxID_ANY );
+        if ( toolid != wxID_ANY )
+        {
+           const wxToolBarToolBase * const tool = FindById(toolid);
+           if ( tool )
+               help = tool->GetLongHelp();
+        }
+
+        // call DoGiveHelp() even if help string is empty to avoid showing the
+        // help for the previously selected tool when another one is selected
+        frame->DoGiveHelp(help, toolid != wxID_ANY);
     }
 
     (void)GetEventHandler()->ProcessEvent(event);
@@ -643,10 +696,8 @@ void wxToolBarBase::UpdateWindowUI(long flags)
 {
     wxWindowBase::UpdateWindowUI(flags);
 
-    // There is no sense in updating the toolbar UI
-    // if the parent window is about to get destroyed
-    wxWindow *tlw = wxGetTopLevelParent( this );
-    if (tlw && wxPendingDelete.Member( tlw ))
+    // don't waste time updating state of tools in a hidden toolbar
+    if ( !IsShown() )
         return;
 
     wxEvtHandler* evtHandler = GetEventHandler() ;
@@ -655,17 +706,21 @@ void wxToolBarBase::UpdateWindowUI(long flags)
           node;
           node = node->GetNext() )
     {
-        int id = node->GetData()->GetId();
+        wxToolBarToolBase * const tool = node->GetData();
+        if ( tool->IsSeparator() )
+            continue;
 
-        wxUpdateUIEvent event(id);
+        int toolid = tool->GetId();
+
+        wxUpdateUIEvent event(toolid);
         event.SetEventObject(this);
 
         if ( evtHandler->ProcessEvent(event) )
         {
             if ( event.GetSetEnabled() )
-                EnableTool(id, event.GetEnabled());
+                EnableTool(toolid, event.GetEnabled());
             if ( event.GetSetChecked() )
-                ToggleTool(id, event.GetChecked());
+                ToggleTool(toolid, event.GetChecked());
 #if 0
             if ( event.GetSetText() )
                 // Set tooltip?
@@ -674,73 +729,33 @@ void wxToolBarBase::UpdateWindowUI(long flags)
     }
 }
 
-#if wxUSE_IMAGE
-
-/*
- * Make a greyed-out image suitable for disabled buttons.
- * This code is adapted from wxNewBitmapButton in FL.
- */
-
-bool wxCreateGreyedImage(const wxImage& src, wxImage& dst)
+#if wxUSE_MENUS
+bool wxToolBarBase::SetDropdownMenu(int toolid, wxMenu* menu)
 {
-    dst = src.Copy();
+    wxToolBarToolBase * const tool = FindById(toolid);
+    wxCHECK_MSG( tool, false, wxT("invalid tool toolid") );
 
-    unsigned char rBg, gBg, bBg;
-    if ( src.HasMask() )
-    {
-        src.GetOrFindMaskColour(&rBg, &gBg, &bBg);
-        dst.SetMaskColour(rBg, gBg, bBg);
-    }
-    else // assuming the pixels along the edges are of the background color
-    {
-        rBg = src.GetRed(0, 0);
-        gBg = src.GetGreen(0, 0);
-        bBg = src.GetBlue(0, 0);
-    }
+    wxCHECK_MSG( tool->GetKind() == wxITEM_DROPDOWN, false,
+                    wxT("menu can be only associated with drop down tools") );
 
-    const wxColour colBg(rBg, gBg, bBg);
-
-    const wxColour colDark = wxSystemSettings::GetColour(wxSYS_COLOUR_3DSHADOW);
-    const wxColour colLight = wxSystemSettings::GetColour(wxSYS_COLOUR_3DHIGHLIGHT);
-
-    // Second attempt, just making things monochrome
-    const int width = src.GetWidth();
-    const int height = src.GetHeight();
-
-    for ( int x = 0; x < width; x++ )
-    {
-        for ( int y = 0; y < height; y++ )
-        {
-            const int r = src.GetRed(x, y);
-            const int g = src.GetGreen(x, y);
-            const int b = src.GetBlue(x, y);
-
-            if ( r == rBg && g == gBg && b == bBg )
-            {
-                // Leave the background colour as-is
-                continue;
-            }
-
-            // Change light things to the background colour
-            wxColour col;
-            if ( r >= (colLight.Red() - 50) &&
-                    g >= (colLight.Green() - 50) &&
-                        b >= (colLight.Blue() - 50) )
-            {
-                col = colBg;
-            }
-            else // Change dark things to really dark
-            {
-                col = colDark;
-            }
-
-            dst.SetRGB(x, y, col.Red(), col.Green(), col.Blue());
-        }
-    }
+    tool->SetDropdownMenu(menu);
 
     return true;
 }
+#endif
 
+#if WXWIN_COMPATIBILITY_2_8
+
+bool wxCreateGreyedImage(const wxImage& in, wxImage& out)
+{
+#if wxUSE_IMAGE
+    out = in.ConvertToGreyscale();
+    if ( out.IsOk() )
+        return true;
 #endif // wxUSE_IMAGE
+    return false;
+}
+
+#endif // WXWIN_COMPATIBILITY_2_8
 
 #endif // wxUSE_TOOLBAR
