@@ -48,7 +48,7 @@
 #endif
 
 #include <oleauto.h>
-#include <shlobj.h>
+#include "wx/msw/wrapshl.h"
 
 #include "wx/msw/ole/oleutils.h"
 
